/* $Header: /wwg/motif/xltmix/RCS/mixer.c,v 1.1 1997/03/16 04:54:50 wwg Exp $
 * Warren W. Gay VE3WWG		Thu Mar  6 22:51:34 1997
 *
 * MIXER FUNCTIONS
 *
 * 	X LessTif Mix :
 * 
 * 	Copyright (C) 1997  Warren W. Gay VE3WWG
 * 
 * This  program is free software; you can redistribute it and/or modify it
 * under the  terms  of  the GNU General Public License as published by the
 * Free Software Foundation version 2 of the License.
 * 
 * This  program  is  distributed  in  the hope that it will be useful, but
 * WITHOUT   ANY   WARRANTY;   without   even  the   implied   warranty  of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details (see enclosed file COPYING).
 * 
 * You  should have received a copy of the GNU General Public License along
 * with this  program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Send correspondance to:
 * 
 * 	Warren W. Gay VE3WWG
 * 	5536 Montevideo Road #17
 *	Mississauga, Ontario L5N 2P4
 * 
 * Email:
 * 	wwg@ica.net			(current ISP of the month :-) )
 * 	bx249@freenet.toronto.on.ca	(backup)
 *
 * $Log: mixer.c,v $
 * Revision 1.1  1997/03/16 04:54:50  wwg
 * Initial revision
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <sys/types.h>
#include <linux/soundcard.h>
#include <asm/io.h>

#include <Xm/Xm.h>

#include "xltmix.h"

static char rcsid[] = "@(#)mixer.c $Revision: 1.1 $";

#ifndef DISABLE_INITCHECK
static int MixIdentified(void);				/* Optional verification of initialization */
#endif

/*
 * Logical Mixer Register Table Entry:
 */
typedef struct {
	unsigned	x;				/* Port index */
	unsigned	mask;				/* Mask */
	unsigned	x2;				/* 0 or index of related register */
	unsigned	shift;				/* Shift count, bits */
	unsigned	deflt;				/* Default value */
} MixTab;

/*
 * Logical Mixer Registers:
 */
static MixTab MixRegs[] = {
	{ 0x00, 0xFF, MIXREG_NONE, 0, 0 },		/* MIXREG_RESET */
	{ 0x04, 0xF0, MIXREG_VOICE_R4, 4, 12 },		/* MIXREG_VOICE_L4 */
	{ 0x04, 0x0F, MIXREG_VOICE_L4, 0, 12 },		/* MIXREG_VOICE_R4 */
	{ 0x0A, 0x07, MIXREG_NONE, 0, 0 },		/* MIXREG_MIC_4 */
	{ 0x22, 0xF0, MIXREG_MSTR_R4, 4, 12 },		/* MIXREG_MSTR_L4 */
	{ 0x22, 0x0F, MIXREG_MSTR_L4, 0, 12 },		/* MIXREG_MSTR_R4 */
	{ 0x26, 0xF0, MIXREG_MIDI_R4, 4, 12 },		/* MIXREG_MIDI_L4 */
	{ 0x26, 0x0F, MIXREG_MIDI_L4, 0, 12 },		/* MIXREG_MIDI_R4 */
	{ 0x28, 0xF0, MIXREG_CD_R4, 4, 0 },		/* MIXREG_CD_L4 */
	{ 0x28, 0x0F, MIXREG_CD_L4, 0, 0 },		/* MIXREG_CD_R4 */
	{ 0x2E, 0xF0, MIXREG_LINE_R4, 4, 0 },		/* MIXREG_LINE_L4 */
	{ 0x2E, 0x0F, MIXREG_LINE_L4, 0, 0 },		/* MIXREG_LINE_R4 */
	{ 0x30, 0xF8, MIXREG_NONE, 3, 24 },		/* MIXREG_MSTR_L5 */
	{ 0x31, 0xF8, MIXREG_NONE, 3, 24 },		/* MIXREG_MSTR_R5 */
	{ 0x32, 0xF8, MIXREG_NONE, 3, 24 },		/* MIXREG_VOICE_L5 */
	{ 0x33, 0xF8, MIXREG_NONE, 3, 24 },		/* MIXREG_VOICE_R5 */
	{ 0x34, 0xF8, MIXREG_NONE, 3, 24 },		/* MIXREG_MIDI_L5 */
	{ 0x35, 0xF8, MIXREG_NONE, 3, 24 },		/* MIXREG_MIDI_R5 */
	{ 0x36, 0xF8, MIXREG_NONE, 3, 0 },		/* MIXREG_CD_L5 */
	{ 0x37, 0xF8, MIXREG_NONE, 3, 0 },		/* MIXREG_CD_R5 */
	{ 0x38, 0xF8, MIXREG_NONE, 3, 0 },		/* MIXREG_LINE_L5 */
	{ 0x39, 0xF8, MIXREG_NONE, 3, 0 },		/* MIXREG_LINE_R5 */
	{ 0x3A, 0xF8, MIXREG_NONE, 3, 0 },		/* MIXREG_MIC_5 */
	{ 0x3B, 0xC0, MIXREG_NONE, 6, 0 },		/* MIXREG_SPKR_2 */
	{ 0x3C, 0x1F, MIXREG_NONE, 0, 0x1F },		/* MIXREG_OUTMIX_SW */
	{ 0x3D, 0x7F, MIXREG_NONE, 0, 0x15 },		/* MIXREG_INMIX_SW_L */
	{ 0x3E, 0x7F, MIXREG_NONE, 0, 0x0B },		/* MIXREG_INMIX_SW_R */
	{ 0x3F, 0xC0, MIXREG_NONE, 6, 0 },		/* MIXREG_IGAIN_L2 */
	{ 0x40, 0xC0, MIXREG_NONE, 6, 0 },		/* MIXREG_IGAIN_R2 */
	{ 0x41, 0xC0, MIXREG_NONE, 6, 0 },		/* MIXREG_OGAIN_L2 */
	{ 0x42, 0xC0, MIXREG_NONE, 6, 0 },		/* MIXREG_OGAIN_R2 */
	{ 0x43, 0x01, MIXREG_NONE, 0, 0 },		/* MIXREG_AGC */
	{ 0x44, 0xF0, MIXREG_NONE, 4, 8 },		/* MIXREG_TREBLE_L4 */
	{ 0x45, 0xF0, MIXREG_NONE, 4, 8 },		/* MIXREG_TREBLE_R4 */
	{ 0x46, 0xF0, MIXREG_NONE, 4, 8 },		/* MIXREG_BASS_L4 */
	{ 0x47, 0xF0, MIXREG_NONE, 4, 8 },		/* MIXREG_BASS_R4 */
};

#ifndef DISABLE_INITCHECK
/*
 * Check List for MixIdentified():
 */
static unsigned CheckList[] = {
	MIXREG_MSTR_L5, MIXREG_MSTR_R5,
	MIXREG_VOICE_L5, MIXREG_VOICE_R5,
	MIXREG_MIDI_L5, MIXREG_MIDI_R5,
	MIXREG_CD_L5, MIXREG_CD_R5,
	MIXREG_LINE_L5, MIXREG_LINE_R5,
	MIXREG_MIC_5,
	MIXREG_SPKR_2,
	MIXREG_OUTMIX_SW,
	MIXREG_INMIX_SW_L, MIXREG_INMIX_SW_R,
	MIXREG_IGAIN_L2, MIXREG_IGAIN_R2,
	MIXREG_OGAIN_L2, MIXREG_OGAIN_R2,
	MIXREG_AGC,
	MIXREG_TREBLE_L4, MIXREG_TREBLE_R4,
	MIXREG_BASS_L4, MIXREG_BASS_R4,
	MIXREG_NONE
};
#endif /* DISABLE_INITCHECK */

/*
 * Mixer initialization 'main' routine:
 */
int
MixInit(void) {

	/*
	 * In order for this program to function, we must have direct access to the
	 * two mixer ports. To do this we need to be either running as root, or have
	 * setuid 'root' on the executable. Once we've got access from the kernel, we
	 * can switch back to ourselves (this is done in the main program in xltmix.c)
	 */
	if ( ioperm(DEV_INDX,2,1) ) {			/* Gain access to our two mixer ports */
		/*
		 * We were not root, nor did we did not have setuid 'root'.
		 */
		sprintf(fatalMsg,"%s: accessing Mixer ports 0x%04X & 0x%04X\n"
			"Try running as root, or setuid root on executable.",
			sys_errlist[errno],
			(unsigned)DEV_INDX,
			(unsigned)DEV_DATA);
		bPortAccessFailed = True;
		return -1;
	}

	/*
	 * This next step is put in for [some] safety and diagnostic purposes: it
	 * will cause a minor click in the speakers, and once you've verified that
	 * this software works in your situation, you might want to disable this
	 * call entirely. Use -DDISPABLE_INITCHECK for this purpose, and recompile.
	 */
#ifndef DISABLE_INITCHECK
	(void) MixIdentified();
#endif
	return 0;
}

#if 0
void							/* Not currently used, but this is how it */
MixerClose(void) {					/* would be done. */
	ioperm(DEV_INDX,2,0);				/* Ignore errors, though there shouldn't be */
}
#endif

/*
 * Raw read of a mixer register:
 */
static unsigned
MixerReadReg(unsigned x) {
	int i;
	int t = 0;

	outb(x,DEV_INDX);				/* Select mixer register to read */
	for ( i=0; i<=7; ++i )				/* Lengthen this loop if your CPU gets too fast */
		t += inb(DEV_DATA);			/* Kill time */
	i = inb(DEV_DATA);				/* Do the real data read */
	return (unsigned) i;				/* Return the data value */
}

/*
 * Raw write of a mixer register:
 */
static void
MixerWriteReg(unsigned x,unsigned b) {
	int i;
	int t = 0;

	outb(x,DEV_INDX);				/* Select mixer register to write */
	for ( i=0; i<=7; ++i )				/* Lengthen this loop if your CPU gets too fast */
		t += inb(DEV_DATA);			/* Kill time */
	outb(b,DEV_DATA);				/* Write the data now */
}

/*
 * Get the LOGICAL mixer register value:
 */
unsigned
MixGetReg(unsigned reg) {
	unsigned b;

	b = MixerReadReg(MixRegs[reg].x) & MixRegs[reg].mask;

	if ( MixRegs[reg].shift > 0 )
		b >>= MixRegs[reg].shift;		/* Shift into position */

	return b;
}

/*
 * Write the LOGICAL mixer register value: Note that in this routine, we have
 * to be concious of paired registers if you want to use the compatibility
 * registers. In the end, none of the paired registers got used, but this
 * code should work for both the newer and older logical registers.
 */
void
MixPutReg(unsigned reg,unsigned b) {
	unsigned t, x;

	if ( (x = MixRegs[reg].x2) != MIXREG_NONE )	/* Is there another register paired with this? */
		t = MixerReadReg(MixRegs[x].x) & MixRegs[x].mask; /* Yes, then get its value first */
	else	t = 0x00;				/* No, we can assume value zero */

	if ( MixRegs[reg].shift > 0 )			/* Does this one get shifted? */
		b <<= MixRegs[reg].shift;		/* Yes, shift left */
	b &= MixRegs[reg].mask;				/* Now mask it */

	t |= b;						/* Or in other paired register, if any */

	MixerWriteReg(MixRegs[reg].x,t);		/* Send to physical register now */
}

/*
 * Mixer initialization check procedure:
 *
 * This function saves the current settings of the mixer, and then applies a mixer RESET.
 * After this hardware reset, if the mixer port has been configured properly, and the
 * proper hardware matches, then all mixer register values after the reset should contain
 * default values which we check for here. If the values don't match defaults, then there
 * are two possibilities:
 *
 *	1.	The mixer port given is invalid:
 *		Action: Recompile with -DDEV_BASE=0xXXXX with the correct port value,
 *			in hex, subsituted for XXXX.
 *
 *	2.	The port is valid, but the chip is perhaps a clone of the original
 *		CT1745 Mixer chip, and does not fully support the default values after
 *		a reset.
 *		Action:	Compile with -DDISABLE_INITCHK
 */
#ifndef DISABLE_INITCHECK
static int
MixIdentified(void) {
	unsigned x;
	unsigned b;
	volatile unsigned z;
	unsigned saved[sizeof CheckList/sizeof CheckList[0]];

	/*
	 * Save all register settings: Note that if the port address is given
	 * incorrectly, this could cause the system to crash here.
	 */
	for ( x=0; CheckList[x] != MIXREG_NONE; ++x )
		saved[x] = MixGetReg(CheckList[x]);

	/*
	 * Reset the mixer, and pause:
	 */
	MixerWriteReg(MixRegs[MIXREG_RESET].x,0x00);	/* Perform Mixer reset */
	for ( x=0; x < 20; ++x )
		z = (unsigned) ((float) 22.0 / 7.0);	/* Waste time for reset */

	/*
	 * Now test reset values of the registers against the known defaults:
	 */
	for ( x=0; CheckList[x] != MIXREG_NONE; ++x ) {
		b = MixGetReg(CheckList[x]);
		if ( b != MixRegs[CheckList[x]].deflt ) {
			sprintf(fatalMsg,"Failed validation of mixer register (x=%u)\n"
				"in %s line %u. Because of this, it seems that you probably\n"
				"do not have a SoundBlaster-16 or SoundBlaster-32 card,\n"
				"or your base port number setting of 0x%04X is incorrect.\n"
				"(bad port numbers can be fatal to your system's integrity!)",
				(unsigned)x,
				__FILE__,__LINE__-7,
				DEV_BASE);
			bPortInvalid = True;
			return -1;
		}
	}

	/*
	 * Restore all register settings:
	 */
	for ( x=0; CheckList[x] != MIXREG_NONE; ++x )
		MixPutReg(CheckList[x],saved[x]);

	return 0;
}
#endif	/* DISABLE_INITCHECK */

/* $Sound$ */
