/* $Header: /wwg/motif/xltmix/RCS/sliders.c,v 1.1 1997/03/16 04:55:52 wwg Exp $
 * Warren W. Gay VE3WWG		Thu Mar  6 21:49:46 1997
 *
 * SLIDERS (Volume Scrollbars)
 *
 * 	X LessTif Mix :
 * 
 * 	Copyright (C) 1997  Warren W. Gay VE3WWG
 * 
 * This  program is free software; you can redistribute it and/or modify it
 * under the  terms  of  the GNU General Public License as published by the
 * Free Software Foundation version 2 of the License.
 * 
 * This  program  is  distributed  in  the hope that it will be useful, but
 * WITHOUT   ANY   WARRANTY;   without   even  the   implied   warranty  of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details (see enclosed file COPYING).
 * 
 * You  should have received a copy of the GNU General Public License along
 * with this  program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Send correspondance to:
 * 
 * 	Warren W. Gay VE3WWG
 * 	5536 Montevideo Road #17
 *	Mississauga, Ontario L5N 2P4
 * 
 * Email:
 * 	wwg@ica.net			(current ISP of the month :-) )
 * 	bx249@freenet.toronto.on.ca	(backup)
 *
 * $Log: sliders.c,v $
 * Revision 1.1  1997/03/16 04:55:52  wwg
 * Initial revision
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <assert.h>

#include <Xm/Xm.h>
#include <Xm/Label.h>
#include <Xm/ScrollBar.h>
#include <Xm/RowColumn.h>
#include <Xm/Frame.h>

#include "xltmix.h"

static char rcsid[] = "@(#)sliders.c $Revision: 1.1 $";

/*
 * The value of a scroll bar changed: Note the new value, and update the mixer
 * chip based upon the new scroll bar setting.
 */
static void
ChangedSlider_CB(Widget w,XtPointer client_data,XtPointer call_data) {
	XmScrollBarCallbackStruct *Ptr = (XmScrollBarCallbackStruct *)call_data;
	DevSliders *pSlider = (DevSliders *)client_data;
	unsigned xLR;

	if ( w == pSlider->Left ) {			/* Left control of a pair? */
		pSlider->vLeft = Ptr->value;		/* Yes, save lefty value */
		xLR = pSlider->xL;			/* and pick the mixer register for left */
	} else	{
		pSlider->vRight = Ptr->value;		/* No, save right handed value */
		xLR = pSlider->xR;			/* and use right handed mixeer register */
	}

	MixPutReg(xLR,(unsigned)Ptr->value);		/* Now put setting into mixer chip */
}

/*
 * This internal routine updates a scroll bar with the current value found in the
 * mixer chip. This is usually only done once at initial startup for each mixer control.
 */
static void
_PutSlider(Widget w,unsigned xLR) {
	unsigned mix_value = MixGetReg(xLR);

	XtVaSetValues(w,XmNvalue,(int)mix_value,NULL);
}

/*
 * Add all slider callbacks:
 */
static void
_AddCallback(Widget w,XtCallbackProc callback_proc,XtPointer client_data) {
	XtAddCallback(w,XmNdecrementCallback,callback_proc,client_data);
	XtAddCallback(w,XmNdragCallback,callback_proc,client_data);
	XtAddCallback(w,XmNincrementCallback,callback_proc,client_data);
	XtAddCallback(w,XmNpageDecrementCallback,callback_proc,client_data);
	XtAddCallback(w,XmNpageIncrementCallback,callback_proc,client_data);
	XtAddCallback(w,XmNtoBottomCallback,callback_proc,client_data);
	XtAddCallback(w,XmNtoTopCallback,callback_proc,client_data);
}

/*
 * Create a pair of left+right sliders, or a singular slider. Does the frames etc.
 */
Widget
CreateSliders(Widget parent,char *class,DevSliders *pSliders,short bStereo,int max,unsigned xL,unsigned xR) {
	Arg al[20];					/* Arg list */
	Cardinal ac;					/* Args count */
	char buf[80];
	static int slider_height = 150;			/* Slider's height in pixels */
	static int slider_width = 20;			/* Slider's width in pixels */
	int slider_size = (max+1) / 4;			/* Compute slider size as 1/4th */

	if ( !slider_size )
		slider_size = 1;			/* Insist on 1 pixel minimum */

	/*
	 * Initialize
	 */
	pSliders->vLeft = pSliders->vRight = 0;
	pSliders->max = max;
	pSliders->xL = xL;
	pSliders->xR = xR;

	/*
	 * The outer frame of the pair of controls
	 */
	sprintf(buf,"%s_frame",class);
	ac = 0;
	XtSetArg(al[ac],XmNmarginHeight,frame_margin); ++ac;
	XtSetArg(al[ac],XmNmarginWidth,frame_margin); ++ac;
	pSliders->Frame = XmCreateFrame(parent,buf,al,ac);
	XtManageChild(pSliders->Frame);

	/*
	 * Vertical container for label + controls:
	 */
	sprintf(buf,"%s_rc",class);
	ac = 0;
	XtSetArg(al[ac],XmNorientation,XmVERTICAL); ++ac;
	XtSetArg(al[ac],XmNpacking,XmPACK_TIGHT); ++ac;
	pSliders->RC = XmCreateRowColumn(pSliders->Frame,buf,al,ac);
	XtManageChild(pSliders->RC);

	/*
	 * Label for the pair of sliders:
	 */
	sprintf(buf,"%s_lbl",class);
	ac = 0;
	pSliders->Lbl = XmCreateLabel(pSliders->RC,buf,al,ac);
	XtManageChild(pSliders->Lbl);

	pSliders->RC2 = 0UL;
	
	/*
	 * Now create stereo pair, or a singular control:
	 */
	if ( bStereo ) {
		/*
		 * Left and right level controls: this row/col container holds them
		 */
		sprintf(buf,"%s_rc2",class);
		ac = 0;
		XtSetArg(al[ac],XmNorientation,XmHORIZONTAL); ++ac;
		XtSetArg(al[ac],XmNpacking,XmPACK_COLUMN); ++ac;
		pSliders->RC2 = XmCreateRowColumn(pSliders->RC,buf,al,ac);
		XtManageChild(pSliders->RC2);
		
		/*
		 * Create the left control:
		 */
		sprintf(buf,"%s_L",class);
		ac = 0;
		XtSetArg(al[ac],XmNheight,slider_height); ++ac;
		XtSetArg(al[ac],XmNwidth,slider_width); ++ac;
		XtSetArg(al[ac],XmNsliderSize,slider_size); ++ac;
		XtSetArg(al[ac],XmNminimum,0); ++ac;
		XtSetArg(al[ac],XmNmaximum,pSliders->max+slider_size); ++ac;
		XtSetArg(al[ac],XmNprocessingDirection,XmMAX_ON_TOP); ++ac;
		pSliders->Left = XmCreateScrollBar(pSliders->RC2,buf,al,ac);
		XtManageChild(pSliders->Left);
		_AddCallback(pSliders->Left,ChangedSlider_CB,(XtPointer)pSliders);

		_PutSlider(pSliders->Left,pSliders->xL);	/* Initialize setting of left control */

		/*
		 * Create the right control:
		 */
		sprintf(buf,"%s_R",class);
		ac = 0;
		XtSetArg(al[ac],XmNheight,slider_height); ++ac;
		XtSetArg(al[ac],XmNwidth,slider_width); ++ac;
		XtSetArg(al[ac],XmNsliderSize,slider_size); ++ac;
		XtSetArg(al[ac],XmNminimum,0); ++ac;
		XtSetArg(al[ac],XmNmaximum,pSliders->max+slider_size); ++ac;
		XtSetArg(al[ac],XmNprocessingDirection,XmMAX_ON_TOP); ++ac;
		pSliders->Right = XmCreateScrollBar(pSliders->RC2,buf,al,ac);
		XtManageChild(pSliders->Right);
		_AddCallback(pSliders->Right,ChangedSlider_CB,(XtPointer)pSliders);

		_PutSlider(pSliders->Right,pSliders->xR);	/* Initialize setting of right control */

	} else	{
		/*
		 * Singular scroll bar:
		 */
		sprintf(buf,"%s_sb",class);
		ac = 0;
		XtSetArg(al[ac],XmNheight,slider_height); ++ac;
		XtSetArg(al[ac],XmNwidth,slider_width); ++ac;
		XtSetArg(al[ac],XmNsliderSize,slider_size); ++ac;
		XtSetArg(al[ac],XmNminimum,0); ++ac;
		XtSetArg(al[ac],XmNmaximum,pSliders->max+slider_size); ++ac;
		XtSetArg(al[ac],XmNprocessingDirection,XmMAX_ON_TOP); ++ac;
		pSliders->Left = XmCreateScrollBar(pSliders->RC,buf,al,ac);
		XtManageChild(pSliders->Left);
		_AddCallback(pSliders->Left,ChangedSlider_CB,(XtPointer)pSliders);

		_PutSlider(pSliders->Left,pSliders->xL);	/* Initialize setting of control */
		pSliders->Right = 0UL;				/* No right control here */
	}

	return pSliders->Frame;
}

/* $Source: /wwg/motif/xltmix/RCS/sliders.c,v $ */
