/* $Header: /wwg/motif/xltmix/RCS/toggle.c,v 1.1 1997/03/16 04:55:59 wwg Exp $
 * Warren W. Gay VE3WWG		Thu Mar  6 21:39:42 1997
 *
 * TOGGLES CODE
 *
 * 	X LessTif Mix :
 * 
 * 	Copyright (C) 1997  Warren W. Gay VE3WWG
 * 
 * This  program is free software; you can redistribute it and/or modify it
 * under the  terms  of  the GNU General Public License as published by the
 * Free Software Foundation version 2 of the License.
 * 
 * This  program  is  distributed  in  the hope that it will be useful, but
 * WITHOUT   ANY   WARRANTY;   without   even  the   implied   warranty  of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details (see enclosed file COPYING).
 * 
 * You  should have received a copy of the GNU General Public License along
 * with this  program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Send correspondance to:
 * 
 * 	Warren W. Gay VE3WWG
 * 	5536 Montevideo Road #17
 *	Mississauga, Ontario L5N 2P4
 * 
 * Email:
 * 	wwg@ica.net			(current ISP of the month :-) )
 * 	bx249@freenet.toronto.on.ca	(backup)
 *
 * $Log: toggle.c,v $
 * Revision 1.1  1997/03/16 04:55:59  wwg
 * Initial revision
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdarg.h>
#include <string.h>
#include <sys/types.h>
#include <assert.h>

#include <Xm/Xm.h>
#include <Xm/Form.h>
#include <Xm/ToggleB.h>
#include <Xm/RowColumn.h>
#include <Xm/Label.h>
#include <Xm/Frame.h>

#include "xltmix.h"

static char rcsid[] = "@(#)toggle.c $Revision: 1.1 $";

static unsigned saved_outmix_sw = 0;

/*
 * This callback is used to toggle a bit in the mixer register, whenever
 * a toggle got changed.
 */
static void
ChangedToggle_CB(Widget w,XtPointer client_data,XtPointer call_data) {
	Tgl *tgl = (Tgl *)client_data;
	XmToggleButtonCallbackStruct *Ptr = (XmToggleButtonCallbackStruct *)call_data;
	unsigned b;
	unsigned bSet = (Ptr->set ? 1 : 0) ^ (tgl->bOpposite ? 1 : 0);

	tgl->bSet = Ptr->set;				/* Update our copy of setting */

	if ( tgl->mixreg >= 0 ) {
		/*
		 * This toggle represents a mixer bit:
		 */
		b = MixGetReg(tgl->mixreg);		/* Get register settings */
		if ( bSet )
			b |= (unsigned)tgl->bits;	/* Set interesting bit true */
		else	b &= ~(unsigned)tgl->bits;	/* Mask out interesting bit */
		MixPutReg(tgl->mixreg,b);		/* Now set value */
	}

	/*
	 * This special case is for toggles in the output switches
	 * group: if any of these get set, then disable the MUTE
	 * toggle (since we're suspending mute effectively)
	 */
	if ( tgl->mixreg == MIXREG_OUTMIX_SW )		/* Disable MUTE if we turn others on */
		XtVaSetValues(*pwMUTE,XmNset,False,NULL);
}

/*
 * This routine is used to take the mixer "Output Mix Switches", and to
 * set or unset the toggle visual.
 */
static void
PutToggle(Tgl *tgl,unsigned flags) {
	Boolean tf = (flags & tgl->bits) != 0 ? True : False;

	tgl->bSet = tf;
	XtVaSetValues(tgl->Toggle,XmNset,tf,NULL);
}

/*
 * This function is used to update all the Output Mix Switch toggles:
 */
static void
PutMixOUTMIX_SW(unsigned new_sw) {
	unsigned short x;
	
	for ( x=0; x<gOutput.n; ++x )
		if ( gOutput.tgl[x].mixreg != MIXREG_NONE ) /* But not the MUTE toggle itself */
			PutToggle(&gOutput.tgl[x],new_sw);  /* Update all others though */
	MixPutReg(MIXREG_OUTMIX_SW,new_sw);		/* Now put new settings into effect */
}

/*
 * This callback is made whenever the MUTE toggle is changed: When set, we save and
 * turn off all other outputs to affect a global "MUTE". If unsetting mute, then
 * we restore all saved settings.
 */
static void
ChangedMuteToggle_CB(Widget w,XtPointer client_data,XtPointer call_data) {
	Tgl *tgl = (Tgl *)client_data;
	XmToggleButtonCallbackStruct *Ptr = (XmToggleButtonCallbackStruct *)call_data;

	if ( (tgl->bSet = Ptr->set) != 0 ) {
		saved_outmix_sw = MixGetReg(MIXREG_OUTMIX_SW);
		PutMixOUTMIX_SW(0x00);			/* Global MUTE */

	} else	PutMixOUTMIX_SW(saved_outmix_sw);	/* Restore settings (and unmute) */
}

/*
 * This messy routine is used to create one toggle, which may be
 * associated with a mixer register.
 */
Widget
CreateToggle(Widget wParent,char *class_name,Tgl *tgl,TglDesc *tdesc) {
	Arg al[20];					/* Arg list */
	Cardinal ac;					/* Args count */
	char buf[80];					/* Temp edit buffer */
	Boolean bSet;					/* New value for the toggle visual */

	if ( tdesc != NULL ) {
		/*
		 * This toggle has initialization info supplied:
		 */
		tgl->mixreg = tdesc->mixreg;		/* Logical mixer register */
		tgl->bits = tdesc->bits;		/* Bit mask */
		if ( tgl->mixreg != MIXREG_NONE )
			tgl->bSet = (MixGetReg(tgl->mixreg) & (unsigned) tgl->bits) ? 1 : 0;
		else	tgl->bSet = tdesc->bits;
		tgl->bOpposite = tdesc->bOpposite;	/* AGC uses opposite logic sense */

	} else	{
		/*
		 * No mixer register, and no init. info supplied: assume defaults.
		 */
		tgl->mixreg = -1;			/* No mixer register associated */
		tgl->bits = tdesc->bits;		/* Initial setting */
		tgl->bSet = (Boolean) tdesc->bits;	/* Set to initial value */
	}

	/*
	 * Determine the visual setting: Note that AGC uses the opposite logic (bOpposite=True)
	 */
	bSet = ((tgl->bSet ? 1 : 0) ^ (tgl->bOpposite ? 1 : 0)) ? True : False;

	/*
	 * Create the toggle:
	 */
	sprintf(buf,"%s_tgl",class_name);
	ac = 0;
	XtSetArg(al[ac],XmNset,bSet); ++ac;
	tgl->Toggle = XmCreateToggleButton(wParent,buf,al,ac);
	XtManageChild(tgl->Toggle);

	/*
	 * Most toggles use callback ChangedToggle_CB(); MUTE gets its own as ChangedMuteToggle_CB()
	 */
	if ( tgl->mixreg != MIXREG_NONE )
		XtAddCallback(tgl->Toggle,XmNvalueChangedCallback,ChangedToggle_CB,(XtPointer)tgl);
	else	XtAddCallback(tgl->Toggle,XmNvalueChangedCallback,ChangedMuteToggle_CB,(XtPointer)tgl);

	return tgl->Toggle;
}

/*
 * Create a whole group of related toggle boxes:
 */
void
CreateToggleGroup(Widget wParent,char *class_name,TglGroup *pTGrp,unsigned n,TglDesc *tdesc) {
	Arg al[20];					/* Arg list */
	Cardinal ac;					/* Args count */
	char buf[80];
	int x;

	/*
	 * Container frame for these toggles:
	 */
	sprintf(buf,"%s_frame",class_name);
	ac = 0;
	XtSetArg(al[ac],XmNmarginHeight,frame_margin); ++ac;
	XtSetArg(al[ac],XmNmarginWidth,frame_margin); ++ac;
	pTGrp->Frame = XmCreateFrame(wParent,buf,al,ac);
	XtManageChild(pTGrp->Frame);

	/*
	 * The row/col container:
	 */
	sprintf(buf,"%s_rc1",class_name);
	ac = 0;
	XtSetArg(al[ac],XmNorientation,XmVERTICAL); ++ac;
	XtSetArg(al[ac],XmNpacking,XmPACK_TIGHT); ++ac;
	pTGrp->RC1 = XmCreateRowColumn(pTGrp->Frame,buf,al,ac);
	XtManageChild(pTGrp->RC1);
	
	/*
	 * Label for this grouping:
	 */
	sprintf(buf,"%s_lbl",class_name);
	ac = 0;
	pTGrp->Lbl = XmCreateLabel(pTGrp->RC1,buf,al,ac);
	XtManageChild(pTGrp->Lbl);

	/*
	 * The row/col for the toggles:
	 */
	sprintf(buf,"%s_rc2",class_name);
	ac = 0;
	XtSetArg(al[ac],XmNorientation,XmVERTICAL); ++ac;
	if ( n > 4 ) {
		XtSetArg(al[ac],XmNnumColumns,2); ++ac;
		XtSetArg(al[ac],XmNpacking,XmPACK_COLUMN); ++ac;
	}
	pTGrp->RC2 = XmCreateRowColumn(pTGrp->RC1,buf,al,ac);
	XtManageChild(pTGrp->RC2);

	pTGrp->n = n;					/* # of toggles in this group */

	/*
	 * Now create all toggles: order of creation is significant.
	 */
	for ( x=0; x<n; ++x ) {
		sprintf(buf,"%s%d",class_name,x);
		CreateToggle(pTGrp->RC2,buf,&pTGrp->tgl[x],&tdesc[x]);
	}
}

/* $Source: /wwg/motif/xltmix/RCS/toggle.c,v $ */
